#include "antlr/TokenStreamSelector.hpp"

/** A token stream MUX (multiplexor) knows about n token streams
 *  and can multiplex them onto the same channel for use by token
 *  stream consumer like a parser.  This is a way to have multiple
 *  lexers break up the same input stream for a single parser.
 */

TokenStreamSelector::TokenStreamSelector()
: input(0)
{
}

TokenStreamSelector::~TokenStreamSelector()
{
}

void TokenStreamSelector::addInputStream(TokenStream* stream, const std::string& key)
{
	inputStreamNames[key] = stream;
}

RefToken TokenStreamSelector::nextToken()
{
	return input->nextToken();
}

void TokenStreamSelector::pop()
{
	TokenStream* stream = streamStack.top();
	streamStack.pop();
	select(stream);
}

void TokenStreamSelector::push(TokenStream* stream)
{
	streamStack.push(input);
	select(stream);
}

void TokenStreamSelector::push(const std::string& sname)
{
	streamStack.push(input);
	select(sname);
}

/** Set the stream without pushing old stream */
void TokenStreamSelector::select(TokenStream* stream)
{
	input = stream;
}

void TokenStreamSelector::select(const std::string& sname)
{
	std::map<std::string,TokenStream*>::const_iterator i = inputStreamNames.find(sname);
	if (i == inputStreamNames.end()) {
		throw std::string("TokenStream ")+sname+" not found";
	}
	input = (*i).second;
}


