#ifndef INC_TokenStreamSelector_hpp__
#define INC_TokenStreamSelector_hpp__

#include "antlr/config.hpp"
#include "antlr/TokenStream.hpp"
#include <map>
#include <stack>

/** A token stream MUX (multiplexor) knows about n token streams
 *  and can multiplex them onto the same channel for use by token
 *  stream consumer like a parser.  This is a way to have multiple
 *  lexers break up the same input stream for a single parser.
 */
class TokenStreamSelector : public TokenStream {
protected:
	/** The set of inputs to the MUX */
	std::map<std::string,TokenStream*> inputStreamNames;

	/** The currently-selected token stream input */
	TokenStream* input;

	/** Used to track stack of input streams */
	std::stack<TokenStream*> streamStack;

public:
	TokenStreamSelector();
	~TokenStreamSelector();

	void addInputStream(TokenStream* stream, const std::string& key);

	RefToken nextToken();

	void pop();

	void push(TokenStream* stream);

	void push(const std::string& sname);

/** Set the stream without pushing old stream */
	void select(TokenStream* stream);

	void select(const std::string& sname);
};

#endif //INC_TokenStreamSelector_hpp__
