/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: InputBuffer.java,v 1.2 2002/03/08 21:53:46 mrobilla Exp $
 */

package at.dms.compiler;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;

import at.dms.util.CharArrayCache;

/**
 * This class allows to find the position of an object in an array.
 */
public class InputBuffer {

  // --------------------------------------------------------------------
  // CONSTRUCTORS
  // --------------------------------------------------------------------

  /**
   * Constructs a new input buffer.
   *
   * @param	name		the name of the file
   * @param	file		the file to read
   * @exception	IOException	an I/O exception occurred
   */
  public InputBuffer(String name, File file) throws IOException {
    FileInputStream	stream = new FileInputStream(file);
    InputStreamReader	reader = new InputStreamReader(stream);

    if (ENV_USE_CACHE) {
      this.data = CharArrayCache.request();
    } else {
      this.data = new char[100000];
    }

    this.size = reader.read(this.data);
    if (this.size == -1) {
      throw new IOException("file " + file + " is empty");
    }
    if (this.size + 1 >= this.data.length) {
      char[]	old = this.data;

      // be conservative this time
      this.data = new char[(int)file.length() + 1];
      System.arraycopy(old, 0, this.data, 0, old.length);

      if (ENV_USE_CACHE) {
	CharArrayCache.release(old);
      }
      this.size += reader.read(this.data, old.length, this.data.length - old.length);
    }
    this.data[this.size++] = ' '; // EOT
    reader.close();
    stream.close();

    this.file = name;
    this.pos = 0;
    this.line = 1;
  }

  /**
   * Constructs a new input buffer.
   *
   * @param	file		the file to read
   * @exception	IOException	an I/O exception occurred
   */
  public InputBuffer(File file) throws IOException {
    this(file.getPath(), file);
  }

  /**
   * Releases cached buffer.
   */
  public void release() {
    if (ENV_USE_CACHE) {
      CharArrayCache.release(this.data);
    }
  }

  // --------------------------------------------------------------------
  // ACCESSORS
  // --------------------------------------------------------------------

  public final boolean isEndOfData() {
    return pos >= size;
  }

  // --------------------------------------------------------------------
  // DATA MEMBERS
  // --------------------------------------------------------------------

  private static final boolean	ENV_USE_CACHE = true;

  public String			file;
  public char[]			data;
  public int			size;
  public int			pos;
  public int			line;
}
