/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CBinaryMethod.java,v 1.2 2002/03/08 21:54:13 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.MethodInfo;

/**
 * This class represents a loaded (already compiled) class method.
 */
public class CBinaryMethod extends CMethod {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs method
   * @param	owner		the owner of this method
   * @param	methodInfo	a method info from a class file
   */
  public CBinaryMethod(CClass owner, MethodInfo methodInfo) {
    super(owner,
	  methodInfo.getModifiers(),
	  methodInfo.getName(),
	  buildReturnType(methodInfo),
	  buildParameterTypes(methodInfo),
	  buildExceptionTypes(methodInfo),
	  methodInfo.isDeprecated());
  }

  private static CType buildReturnType(MethodInfo methodInfo) {
    CType[]	types = CType.parseMethodSignature(methodInfo.getSignature());

    return types[types.length - 1];
  }

  private static CType[] buildParameterTypes(MethodInfo methodInfo) {
    CType[]	signature = CType.parseMethodSignature(methodInfo.getSignature());
    CType[]	paramTypes = new CType[signature.length - 1];

    for (int i = 0; i < paramTypes.length; i++) {
      paramTypes[i] = signature[i];
    }

    return paramTypes;
  }

  private static CClassType[] buildExceptionTypes(MethodInfo methodInfo) {
    String[]	exceptions = methodInfo.getExceptions();

    if (exceptions == null) {
      return new CClassType[0];
    } else {
      CClassType[]	types = new CClassType[exceptions.length];

      for (int i = 0; i < exceptions.length; i++) {
	types[i] = CClassType.lookup(exceptions[i]);
      }
      return types;
    }
  }
}
