/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CExpressionContext.java,v 1.2 2002/03/08 21:54:15 mrobilla Exp $
 */

package at.dms.kjc;

/**
 * This class represents a local context during checkBody
 * It follows the control flow and maintain informations about
 * variable (initialized, used, allocated), exceptions (thrown, catched)
 * It also verify that context is still reachable
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree
 * @see CContext
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CConstructorContext
 * @see CInitializerContext
 * @see CBodyContext
 * @see CBlockContext
 */
public class CExpressionContext extends CContext {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a block context, it supports local variable allocation
   * throw statement and return statement
   * This starts a new context in a method
   * @param	parent		the parent context, it must be different
   *				than null except if called by the top level
   */
  public CExpressionContext(CBodyContext parent) {
    super(parent);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns the parent context.
   */
  public CBodyContext getBodyContext() {
    return (CBodyContext)parent;
  }

  /**
   * Set this context as a left side in an assignment, so access to vars
   * may be uninitialized
   * @param	leftSide		set the side of current code flow
   */
  public void setLeftSide(boolean leftSide) {
    this.leftSide = leftSide;
  }

  /**
   * @return	true if this context is on a left side of an assignement
   */
  public boolean isLeftSide() {
    return leftSide;
  }

  /**
   * set this context as a left side in an assignment, so access to vars
   * may be uninitialized
   * @param	leftSide		set the side of current code flow
   */
  public void setDiscardValue(boolean discardValue) {
    this.discardValue = discardValue;
  }

  /**
   * @return	true if the result value of this expression is discarded
   */
  public boolean discardValue() {
    return discardValue;
  }

  // ----------------------------------------------------------------------
  // PRIVATE UTILITIES
  // ----------------------------------------------------------------------

  protected	boolean		discardValue;
  protected	boolean		leftSide;
}
