/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CField.java,v 1.2 2002/03/08 21:54:16 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.FieldInfo;
import at.dms.classfile.FieldRefInstruction;

/**
 * This class represents an exported member of a class (fields)
 */
public abstract class CField extends CMember {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a field export
   * @param	owner		the owner of this field
   * @param	modifiers	the modifiers on this field
   * @param	ident		the name of this field
   * @param	type		the type of this field
   * @param	deprecated	is this field deprecated ?
   */
  public CField(CClass owner, int modifiers, String ident, CType type, boolean deprecated) {
    super(owner, modifiers, ident, deprecated);
    this.type = type;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * @return	the interface
   */
  public CField getField() {
    return this;
  }

  /**
   * @return the type of this field
   */
  public CType getType() {
    return type;
  }

  /**
   * @param	value		the value known at third pass
   */
  public void setValue(JExpression value) {
    this.value = value;
  }

  /**
   * @return	the value of initializer or null
   */
  public JExpression getValue() {
//!!!TEST
{
  if (value != null && ! value.getType().equals(type)) {
    System.err.println("<<< get " + getPrefixName() + "." + getIdent() + ": bad value type " + value.getType() + "(" + type + " expected)");
  }
}
//!!!TEST
    return value;
  }

  // ----------------------------------------------------------------------
  // GENERATE CLASSFILE INFO
  // ----------------------------------------------------------------------

  /**
   * Generates a sequence of bytecodes to load
   * @param	code		the code list
   */
  public void genLoad(CodeSequence code) {
    code.plantInstruction(new FieldRefInstruction(isStatic() ? opc_getstatic : opc_getfield,
						  getPrefixName(),
						  getIdent(),
						  getType().getSignature()));
  }

  /**
   * Generates a sequence of bytecodes to load
   * @param	code		the code list
   */
  public void genStore(CodeSequence code) {
    code.plantInstruction(new FieldRefInstruction(isStatic() ? opc_putstatic : opc_putfield,
						  getPrefixName(),
						  getIdent(),
						  getType().getSignature()));
  }

  // ----------------------------------------------------------------------
  // GENERATE CLASSFILE INFO
  // ----------------------------------------------------------------------

  /**
   * Generate the code in a class file
   */
  public FieldInfo genFieldInfo() {
    Object		value = null;

    if (this.value != null &&
	this.value.isConstant() &&
	isFinal() &&
	isStatic() &&
	!(this.value instanceof JNullLiteral)) {
      value = this.value.getLiteral().getValue();
      if (this.value.getType().isOrdinal() && !(value instanceof Long)) {
	value = new Integer(this.value.getOrdinalLiteral().numberValue().intValue());
      } else if (this.value.getType() == CStdType.Float) {
	value = new Float(((Number)value).floatValue());
      }
    }

    return new FieldInfo((short)getModifiers(),
			 getIdent(),
			 type.getSignature(),
			 value,
			 isDeprecated(),
			 getIdent().indexOf('$') >= 0);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private	CType		type;
  private	JExpression	value;
}
