/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CFieldTable.java,v 1.2 2002/03/08 21:54:16 mrobilla Exp $
 */

package at.dms.kjc;


import java.util.Enumeration;
import java.util.Hashtable;

import at.dms.classfile.FieldInfo;

/**
 * This class encapsulates the fields of a class.
 */
public class CFieldTable extends at.dms.util.Utils {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a field table
   *
   */
  public CFieldTable(CField[] array, Hashtable hashed) {
    this.array = array;
    this.hashed = hashed;
    assert((array != null) || (hashed != null));
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Returns the field with specified name
   * @param	ident		the simple name of the field
   */
  public CField lookup(String ident) {
    if (hashed != null) {
      return (CField)hashed.get(ident);
    } else {
      for (int i = 0; i < array.length; i++) {
	if (array[i].getIdent() == ident) {
	  return array[i];
	}
      }
      return null;
    }
  }

  /**
   * Returns the number of fields.
   */
  public int size() {
    return hashed == null ? array.length : hashed.size();
  }

  /**
   * Adds a field.
   */
  public void add(CSourceField field) {
    field.setPosition(hashed.size());
    hashed.put(field.getIdent(), field);
    //!!! remove array
  }

  /**
   * Returns the field infos
   */
  public FieldInfo[] buildFieldInfo() {
    FieldInfo[]		fields;

    if (hashed != null) {
      fields = new FieldInfo[hashed.size()];

      for (Enumeration enum = hashed.elements(); enum.hasMoreElements(); ) {
	CSourceField	field = (CSourceField)enum.nextElement();

	fields[field.getPosition()] = field.genFieldInfo();
      }
    } else {
      fields = new FieldInfo[array.length];

      for (int i = 0; i < array.length; i++) {
	fields[i] = array[i].genFieldInfo();	
      }
    }

    return fields;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private CField[]		array;
  private Hashtable		hashed;
}
