/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CLabeledContext.java,v 1.2 2002/03/08 21:54:17 mrobilla Exp $
 */

package at.dms.kjc;

/**
 * This class represents a local context during checkBody
 * It follows the control flow and maintain informations about
 * variable (initialised, used, allocated), exceptions (thrown, catched)
 * It also verify that context is still reachable
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CContext
 */
public class CLabeledContext extends CAbstractBodyContext {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a block context, it supports local variable allocation
   * throw statement and return statement
   * @param	parent		the parent context, it must be different
   *				than null except if called by the top level
   */
  CLabeledContext(CBodyContext parent, JLabeledStatement self) {
    super(parent);

    this.self = self;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (NEW CONTEXT)
  // ----------------------------------------------------------------------

  public CBodyContext createBodyContext() {
    return new CBodyContext(this);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (SWITCHES)
  // ----------------------------------------------------------------------

  /**
   * Returns the statement with the specified label.
   */
  public JLabeledStatement getLabeledStatement(String label) {
    if (label.equals(self.getLabel())) {
      return self;
    } else {
      return ((CBodyContext)parent).getLabeledStatement(label);
    }
  }


  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private final JLabeledStatement	self;
}
