/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CNullType.java,v 1.2 2002/03/08 21:54:19 mrobilla Exp $
 */

package at.dms.kjc;

/**
 * This class represents null class type in the type structure
 */
public class CNullType extends CClassType {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a class type
   */
  public CNullType() {
    super(new CBadClass("<NULL TYPE>"));
  }

  // ----------------------------------------------------------------------
  // INTERFACE CHECKING
  // ----------------------------------------------------------------------

  /**
   * isAssignableTo
   * @param dest  the destination type
   * @return  assignment is valid ?
   */
  public boolean isAssignableTo(CType dest) {
    return dest.isReference();
  }

  /**
   * isCastableTo
   * @param dest  the destination type
   * @return  cast is valid ?
   */
  public boolean isCastableTo(CType dest) {
    return dest.isReference();
  }

  /**
   * implicitlyCastTo
   * @param	dest	the expression type to cast to
   * @return	cast is valid ?
   */
  public boolean implicitlyCastTo(CType dest) {
    return dest.isReference();
  }

  /**
   * get the object, if this type was never checked (reads from class files)
   * check it!
   */
  protected CClass getObject() {
    return CStdType.Object.getCClass();
  }

  /**
   * Returns the class object associated with this type
   *
   * If this type was never checked (read from class files)
   * check it!
   *
   * @return the class object associated with this type
   */
  public CClass getCClass() {
    return CStdType.Object.getCClass();
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Transforms this type to a string
   */
  public String toString() {
    return "null-literal";
  }

  /**
   * Returns the size used in stack by value of this type
   */
  public int getSize() {
    return 1;
  }
}
