/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CParseClassContext.java,v 1.2 2002/03/08 21:54:19 mrobilla Exp $
 */

package at.dms.kjc;

import java.util.Vector;
import java.util.Stack;

public class CParseClassContext {
  public static CParseClassContext getInstance() {
    return stack.size() == 0 ?
      new CParseClassContext() :
      (CParseClassContext)stack.pop();
  }

  public void release() {
    release(this);
  }

  public static void release(CParseClassContext context) {
    context.clear();
    stack.push(context);
  }

  private CParseClassContext(){}

  private void clear() {
    fields.setSize(0);
    methods.setSize(0);
    inners.setSize(0);
    body.setSize(0);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (ADD)
  // ----------------------------------------------------------------------

  public void addFieldDeclaration(JFieldDeclaration decl) {
    fields.addElement(decl);
    body.addElement(decl);
  }

  public void addMethodDeclaration(JMethodDeclaration decl) {
    methods.addElement(decl);
  }
 
  public void addInnerDeclaration(JTypeDeclaration decl) {
    inners.addElement(decl);
  }

  public void addBlockInitializer(JClassBlock block) {
    body.addElement(block);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (GET)
  // ----------------------------------------------------------------------

  public JFieldDeclaration[] getFields() {
    JFieldDeclaration[]	temp = new JFieldDeclaration[fields.size()];

    fields.copyInto(temp);

    return temp;
  }

  public JMethodDeclaration[] getMethods() {
    JMethodDeclaration[]	temp = new JMethodDeclaration[methods.size()];

    methods.copyInto(temp);

    return temp;    
  }

  public JTypeDeclaration[] getInnerClasses() {
    JTypeDeclaration[]	temp = new JTypeDeclaration[inners.size()];

    inners.copyInto(temp);

    return temp;    
  }

  public JPhylum[] getBody() {
    JPhylum[]	temp = new JPhylum[body.size()];

    body.copyInto(temp);

    return temp;
  }

  // ----------------------------------------------------------------------
  // UTILITIES
  // ----------------------------------------------------------------------

  public static CClassType[] vectorToClassType(Vector aVect) {
    CClassType[] temp = new CClassType[aVect.size()];

    for (int i = aVect.size() - 1; i >= 0; i--) {
      temp[i] = CClassType.lookup((String)aVect.elementAt(i));
    }

    return temp;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private Vector fields = new Vector();
  private Vector methods = new Vector();
  private Vector inners = new Vector();
  private Vector body = new Vector();

  private static Stack stack = new Stack();
}
