/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CTopLevel.java,v 1.2 2002/03/08 21:54:21 mrobilla Exp $
 */

package at.dms.kjc;

import java.util.Hashtable;
import java.util.Enumeration;
import at.dms.classfile.ClassInfo;
import at.dms.compiler.Compiler;

/**
 * This class implements the conceptual directory structure for .class files
 */
public final class CTopLevel extends at.dms.util.Utils {

  // ----------------------------------------------------------------------
  // LOAD CLASS
  // ----------------------------------------------------------------------

  /**
   * Loads class definition from .class file
   */
  public static CClass loadClass(String name) {
    CClass		cl = (CClass)allLoadedClasses.get(name);

    if (cl != null) {
      // look in cache
      return cl != CClass.CLS_UNDEFINED ? cl : null;
    } else {
      ClassInfo		file = at.dms.classfile.ClassPath.getClassInfo(name, true);

      cl = file == null ? CClass.CLS_UNDEFINED : new CBinaryClass(FROMCLASSES, file);
      allLoadedClasses.put(name, cl);

      return cl;
    }
  }

  /**
   * Reloads the specified class if this class has been defined during a previous compilation session
   */
  public static CClass reloadClass(CClass clazz) {
    if (clazz == null) {
      return null;
    } else if (clazz.getCompiler() == compiler) {
      // Same compilation session: keep it
      return clazz;
    } else {
      // Loaded in a previous compilation session: reload it
      return loadClass(clazz.getQualifiedName());
    }
  }

  /**
   * @return  false if name exists for source class as source class
   *          in an other file
   * @param CClass a class to add (must be a CSourceClass)
   */
  public static boolean addSourceClass(CClass cl) {
    assert(cl instanceof CSourceClass);

    CClass	last = (CClass)allLoadedClasses.put(cl.getQualifiedName(), cl);
    return (last == null) ||
      (cl.getOwner() != null) ||
      !(last instanceof CSourceClass) ||
      last.getSourceFile() == cl.getSourceFile();
  }

  /**
   * @return a class file that contain the class named name
   * @param name the name of the class file
   */
  public static boolean hasClassFile(String name) {
    // already loaded or parsed ?
    if (allLoadedClasses.containsKey(name)) {
      return true;
    } else {
      return at.dms.classfile.ClassPath.hasClassFile(name);
    }
  }

  /**
   * Removes all source classes
   */
  public static void initSession(Compiler compiler) {
    CTopLevel.compiler = compiler;

    Enumeration		enum = allLoadedClasses.elements();
    while (enum.hasMoreElements()) {
      CClass clazz = (CClass)enum.nextElement();
      if (clazz instanceof CSourceClass) {
	allLoadedClasses.remove(clazz.getQualifiedName());
      }
    }
  }

  /**
   *
   */
  public static void endSession(Compiler compiler) {
    CTopLevel.compiler = null;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private static Compiler	compiler;

  private static final Compiler FROMCLASSES = new Compiler() {
      public boolean run(String[] args) {
	return true;
      }
      public void reportTrouble(at.dms.compiler.PositionedError a) {
      }
      public boolean parseComments() {
	return true;
      }
      public boolean verboseMode() {
	return false;
      }
    };

  private static Hashtable	allLoadedClasses = new Hashtable(2000);
}
