/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: CTryContext.java,v 1.2 2002/03/08 21:54:21 mrobilla Exp $
 */

package at.dms.kjc;

import java.util.Hashtable;
import java.util.Enumeration;

import at.dms.compiler.TokenReference;

/**
 * This class represents a local context during checkBody
 * It follows the control flow and maintain informations about
 * variable (initialised, used, allocated), exceptions (thrown, catched)
 * It also verify that context is still reachable
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CContext
 */
public class CTryContext extends CAbstractBodyContext {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a block context, it supports local variable allocation
   * throw statement and return statement
   * @param	parent		the parent context, it must be different
   *				than null except if called by the top level
   */
  CTryContext(CBodyContext parent) {
    super(parent);

    olds = parent.getThrowables();
    throwables = new Hashtable();
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (NEW CONTEXT)
  // ----------------------------------------------------------------------

  public CBodyContext createBodyContext() {
    return new CBodyContext(this);
  }

  public void close(TokenReference ref) {
    if (throwables != null && throwables.size() > 0) {
      Enumeration	enum = throwables.elements();
      
      while (enum.hasMoreElements()) {
	Object	o = enum.nextElement();
	
	olds.put(o, o);
      }
    }
    throwables = olds;

    super.close(ref);
  }

  // ----------------------------------------------------------------------
  // ACCESSORS (THROWABLES)
  // ----------------------------------------------------------------------

  public void addThrowables(Hashtable news) {
    if (news.size() > 0) {
      Enumeration	enum = news.elements();
      
      while (enum.hasMoreElements()) {
	Object	o = enum.nextElement();
	
	throwables.put(o, o);
      }
    }
  }

  public void setThrowables(Hashtable news) {
    throwables = news;
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private Hashtable	olds;
}
