/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JCastExpression.java,v 1.2 2002/03/08 21:54:25 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.ClassRefInstruction;
import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a cast expression '((byte)2)'
 */
public class JCastExpression extends JExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Constructs a node in the parsing tree.
   * This method is directly called by the parser.
   * @param	where	the line of this node in the source code
   * @param	expr	the expression to be casted
   * @param	dest	the type of this expression after cast
   */
  public JCastExpression(TokenReference where, JExpression expr, CType dest) {
    super(where);
    this.expr = expr;
    this.dest = dest;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression.
   *
   * @return the type of this expression
   */
  public CType getType() {
    return dest;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param	context		the actual context of analyse
   * @return	a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    expr = expr.checkExpression(context);

    try {
      dest.checkType(context);
    } catch (UnpositionedError e) {
      throw e.addPosition(getTokenReference());
    }

    check(context, expr.getType().isCastableTo(dest), KjcMessages.CAST_CANT, expr.getType(), dest);

    if (!expr.getType().isPrimitive() && expr.getType().implicitlyCastTo(dest) && expr.getType() != CStdType.Null) {
      context.reportTrouble(new CWarning(getTokenReference(), KjcMessages.UNNECESSARY_CAST, expr.getType(), dest));
    }

    if (expr.isConstant() && expr.getType().isPrimitive()) {
      return expr.convertType(dest, context);
      /*
       * !!! graf 000520 remove
      if (expr.getType().isNumeric()) {
	return JLiteral.createLiteral(dest, expr.getNumberLiteral().getValue());
      } else if (!expr.getType().isReference()) {
	fail();
	return expr;
      }
       * !!! graf 000520 remove
      */
    }

    if (!dest.isAssignableTo(expr.getType())) { // WAS: expr.getType() / dest
      return expr.convertType(dest, context);
    }

    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitCastExpression(this, expr, dest);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    expr.genCode(code);

    if (dest.isNumeric()) {
      ((CNumericType)expr.getType()).genCastTo((CNumericType)dest, code);
    } else if (dest instanceof CClassType) {
      code.plantInstruction(new ClassRefInstruction(opc_checkcast, ((CClassType)dest).getQualifiedName()));
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected JExpression		expr;
  protected CType		dest;
}
