/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JCatchClause.java,v 1.2 2002/03/08 21:54:25 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.HandlerInfo;
import at.dms.compiler.CWarning;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents a parameter declaration in the syntax tree
 */
public class JCatchClause extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	ident		the name of this variable
   * @param	initializer	the initializer
   */
  public JCatchClause(TokenReference where, JFormalParameter exception, JBlock body) {
    super(where);

    this.exception = exception;
    this.body = body;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * getType
   * @return	the type of exception catched by this clause
   */
  public CClassType getType() {
    return (CClassType)exception.getType();
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  public boolean isChecked() {
    return checked;
  }

    /**
     * Check statement.
     * @param	context		the actual context of analyse
     * @exception	PositionedError		if the check fails
     */
    public void checkInterface(CBodyContext context) throws PositionedError
    {
	exception.checkInterface(context.getClassContext()); // will be checked after    
    }

  /**
   * Check statement.
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public void checkBody(CBodyContext context) throws PositionedError {
    checked = true;
    exception.checkInterface(context.getClassContext()); // will be checked after

    CBlockContext	block = context.createBlockContext(1);
    CBodyContext	inside = block.createBodyContext();

    inside.setReachable(true);
    exception.checkBody(inside);
    body.checkStatement(inside);

    inside.close(getTokenReference());
    block.close(getTokenReference());

    if (body.getBody().length == 0) {
      context.reportTrouble(new CWarning(getTokenReference(),
					 KjcMessages.EMPTY_CATCH_BLOCK));
    }
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitCatchClause(this, exception, body);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    code.setLineNumber(getTokenReference().getLine());

    catchPC = code.getPC();
    exception.genStore(code);
    body.genCode(code);
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genExceptionHandler(CodeSequence code, int startPC, int endPC) {
    code.addExceptionHandler(new HandlerInfo(code.getInstructionAt(startPC),
					     code.getInstructionAt(endPC),
					     code.getInstructionAt(catchPC),
					     exception.getType().getCClass().getQualifiedName()));
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private JFormalParameter	exception;
  private JBlock		body;
  private boolean		checked;
  private int			catchPC;
}
