/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JCheckedExpression.java,v 1.2 2002/03/08 21:54:25 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.compiler.TokenReference;

/**
 * to add a portion of already checked code into a code to check,
 * the checkExpression will break the code checking and return the
 * the children
 */
public final class JCheckedExpression extends JExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

 /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   */
  public JCheckedExpression(TokenReference where, JExpression checked) {
    super(where);
    this.checked = checked;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression (called after parsing)
   * @return the type of this expression
   */
  public CType getType() {
    return checked.getType();//throw new RuntimeException("CHECK ME BEFORE AND YOU WONT SEE ME ANYMORE");
  }

  /**
   * @return true if this expression is constant
   */
  public boolean isConstant() {
    throw new RuntimeException("CHECK ME BEFORE AND YOU WONT SEE ME ANYMORE");
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public JExpression checkExpression(CExpressionContext context) {
    return checked; // already checked
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    throw new RuntimeException("CHECK ME BEFORE AND YOU WONT SEE ME ANYMORE");
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    throw new RuntimeException("CHECK ME BEFORE AND YOU WONT SEE ME ANYMORE");
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  private JExpression checked;
}
