/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JNewAnonymousClassExpression.java,v 1.2 2002/03/08 21:54:33 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;

/**
 * This class represents a new allocation expression 'new Toto(1){}'
 */
public class JNewAnonymousClassExpression extends JNewObjectExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	objectType	the type of this object allocator
   * @param	explicitThis	a TypeName expression
   * @param	params		parameters to be passed to constructor
   */
  public JNewAnonymousClassExpression(TokenReference where,
				      CClassType objectType,
				      JExpression explicitThis,
				      JExpression[] params,
				      JClassDeclaration decl) {
    super(where, objectType, explicitThis, params);

    this.decl = decl;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  protected CType[] checkTypes(CExpressionContext context) throws PositionedError {
    CType[]	argsType = super.checkTypes(context);
    CClass	owner = context.getClassContext().getCClass();    

    if (type.getCClass().isInterface()) {
      decl.setInterfaces(new CClassType[] { type });
    } else {
      decl.setSuperClass(type.getCClass().getQualifiedName());
    }

    decl.generateInterface((Main)context.getCompiler(),
			   owner, 
			   owner.getQualifiedName() + "$" + context.getClassContext().getNextSyntheticIndex() + "$");

    // add implicit constructor
    JConstructorDeclaration	cstr;
    CMethod			superCstr;
    try {
      superCstr = type.getCClass().lookupMethod(JAV_CONSTRUCTOR, argsType);
    } catch (UnpositionedError cue) {
      throw cue.addPosition(getTokenReference());
    }
    if (superCstr == null) {
      throw new CMethodNotFoundError(getTokenReference(), null, type.toString(), argsType);
    }

    CType[]		parameters = superCstr.getParameters();
    JFormalParameter[]	fparams = new JFormalParameter[parameters.length];
    CClassType[]	throwables = superCstr.getThrowables();

    for (int i = 0; i < parameters.length; i++) {
      fparams[i] = new JFormalParameter(getTokenReference(),
				       JLocalVariable.DES_GENERATED,
				       parameters[i],
				       "dummy" + i,
				       true);
    }

    JExpression[]		checkedParams = new JExpression[params.length];

    for (int i = 0; i < params.length; i++) {
      checkedParams[i] = new JLocalVariableExpression(getTokenReference(),
						      fparams[i]);
						      //params[i].getTokenReference(), params[i]);
    }

    cstr = new JConstructorDeclaration(getTokenReference(),
				       ACC_PUBLIC,
				       decl.getCClass().getIdent(),
				       fparams,
				       throwables,
				       new JConstructorBlock(getTokenReference(), new JStatement[] {
					 new JExpressionStatement(getTokenReference(),
								  new JExplicitConstructorInvocation(getTokenReference(), null, "super", checkedParams), null)}),
				       null,
				       null);
    decl.setDefaultConstructor(cstr);
    decl.checkInterface(context.getCompilationUnit());
    decl.checkInitializers(context);
    decl.checkTypeBody(context);

    type = decl.getCClass().getType();

    return argsType;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitNewObjectExpression(this, type, params, decl);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  protected	JClassDeclaration	decl;
}
