/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JSwitchLabel.java,v 1.2 2002/03/08 21:54:37 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;
import at.dms.compiler.UnpositionedError;
import at.dms.util.MessageDescription;

/**
 * This class represents a parameter declaration in the syntax tree
 */
public class JSwitchLabel extends JPhylum {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

  /**
   * Construct a node in the parsing tree
   * This method is directly called by the parser
   * @param	where		the line of this node in the source code
   * @param	expr		the expression (null if default label)
   */
  public JSwitchLabel(TokenReference where, JExpression expr) {
    super(where);

    this.expr = expr;
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * @return	true if this label is a "default:" label
   */
  public boolean isDefault() {
    return expr == null;
  }

  /**
   * @return	the value of this label
   */
  public Integer getLabel() {
    return label;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check statement.
   *
   * @param	context		the actual context of analyse
   * @exception	PositionedError		if the check fails
   */
  public void checkBody(CSwitchBodyContext switchContext, CBodyContext context) throws PositionedError {
    try {
      if (expr != null) {
	expr = expr.checkExpression(context.createExpressionContext());
	check(context,
	      expr.getType().implicitlyCastTo(CStdType.Integer),
	      KjcMessages.SWITCH_LABEL_EXPR_NOTINT, expr.getType());

	check(context, expr.isConstant(), KjcMessages.SWITCH_LABEL_EXPR_NOTCONST);
	switchContext.addLabel(expr.getOrdinalLiteral());
	label = new Integer(expr.getOrdinalLiteral().numberValue().intValue());
      } else {
	switchContext.addDefault();
      }
    } catch (UnpositionedError cue) {
      throw cue.addPosition(getTokenReference());
    }
  }

  /**
   * Adds a compiler error.
   * @param	context		the context in which the error occurred
   * @param	key		the message ident to be displayed
   * @param	params		the array of parameters
   *
   */
  protected void fail(CContext context, MessageDescription key, Object[] params)
    throws PositionedError
  {
    throw new CLineError(getTokenReference(), key, params);
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    p.visitSwitchLabel(this, expr);
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private Integer		label;
  private JExpression		expr;
}
