/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: JUnaryPromote.java,v 1.2 2002/03/08 21:54:39 mrobilla Exp $
 */

package at.dms.kjc;

import at.dms.classfile.ClassRefInstruction;
import at.dms.compiler.PositionedError;

/**
 * This class convert arithmetics expression from types to types
 */
public class JUnaryPromote extends JExpression {

  // ----------------------------------------------------------------------
  // CONSTRUCTORS
  // ----------------------------------------------------------------------

 /**
   * Construct a node in the parsing tree
   * @param	where		the line of this node in the source code
   */
  public JUnaryPromote(JExpression expr, CType type) {
    super(expr.getTokenReference());

    this.expr = expr;
    this.type = type;

    if (!expr.isAssignableTo(type)) {
      needCheck = true;
    }
  }

  // ----------------------------------------------------------------------
  // ACCESSORS
  // ----------------------------------------------------------------------

  /**
   * Compute the type of this expression (called after parsing)
   * @return the type of this expression
   */
  public CType getType() {
    return type;
  }

  // ----------------------------------------------------------------------
  // CODE CHECKING
  // ----------------------------------------------------------------------

  /**
   * Check expression and evaluate and alter context
   * @param context the actual context of analyse
   * @return  a pure java expression including promote node
   * @exception	PositionedError		if the check fails
   * @exception	UnpositionedError	this error will be positioned soon
   */
  public JExpression checkExpression(CExpressionContext context) throws PositionedError {
    if (type.equals(CStdType.String) && expr.getType().isReference() && !(expr instanceof JNullLiteral)) {
      JNameExpression name = new JNameExpression(getTokenReference(),
						 new JCheckedExpression(getTokenReference(), expr),
						 "toString");

      return new JMethodCallExpression(getTokenReference(), name, JExpression.EMPTY).checkExpression(context);
    }

    return this;
  }

  // ----------------------------------------------------------------------
  // CODE GENERATION
  // ----------------------------------------------------------------------

  /**
   * Accepts the specified visitor
   * @param	p		the visitor
   */
  public void accept(KjcVisitor p) {
    if (needCheck) {
      p.visitUnaryPromoteExpression(this, expr, getType());
    } else {
      expr.accept(p);
    }
  }

  /**
   * Generates a sequence of bytescodes
   * @param	code		the code list
   */
  public void genCode(CodeSequence code) {
    expr.genCode(code);

    if (type.isNumeric()) {
      ((CNumericType)expr.getType()).genCastTo((CNumericType)type, code);
    } else if (needCheck) {
      code.plantInstruction(new ClassRefInstruction(opc_checkcast,
						    ((CClassType)type).getQualifiedName()));
    }
  }

  // ----------------------------------------------------------------------
  // PRIVATE DATA MEMBERS
  // ----------------------------------------------------------------------

  private JExpression		expr;
  private CType			type;
  private boolean		needCheck;
}
