/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import java.io.PrintWriter;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

/** Represents a scope for exception descriptors.
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:44 $
  */
public class ExceptionBlock
{
    private Vector aElements;
    
    /** Builds a new, empty Exception Block.
      */
    public ExceptionBlock()
    { aElements = new Vector(); }

    /** Adds an exception descriptor to the block
      * @param pElement The element to add.
      */
    public void addElement( ExceptionElement pElement )
    {
	if( aElements.contains( pElement ) == false )
	    aElements.addElement( pElement );
    }
    
    /** Returns all the exceptions that can arise from all the containers;
      * This does not necessarily correspond to the exceptions contained
      * in the block, since exceptions that are caught in a catch clause
      * are not listed.
      */
    public Set getExceptions()
    {
	HashSet lReturn = new HashSet();

	for( int i = 0; i < aElements.size(); i++ )
	{
	    Set lTempExceptions = ((ExceptionElement)aElements.elementAt(i)).getExceptions();
	    lReturn.addAll( lTempExceptions );
	}
	
	return lReturn;
    }

    /** Writes the exception block out to a file.
      * @param pFile The PrintWriter to write the block to.
      * @param pPrefix The indentation.
      */
    public void write( PrintWriter pFile, String pPrefix )
    {
	pFile.println( pPrefix + "{");
	
	for( int i = 0; i < aElements.size(); i++ )
	    ((ExceptionElement)aElements.elementAt(i)).write( pFile, pPrefix + ExceptionElement.kTab );
	
	pFile.println( pPrefix + "}");
    }

    /** Writes the exceptions block out to a file, prefixed by another exception block.
      * @param pFile The PrintWriter to write the block to.
      * @param pPrefix The indentation.
      * @param pInit The Block to add as a prefix to this block.
      */
    public void write( PrintWriter pFile, String pPrefix, ExceptionBlock pInit )
    {
	pFile.println( pPrefix + "{" );
	
	for( int i = 0; i < pInit.aElements.size(); i++ )
	    ((ExceptionElement)pInit.aElements.elementAt(i)).write( pFile, pPrefix + ExceptionElement.kTab );
	
	for( int i = 0; i < aElements.size(); i++ )
	    ((ExceptionElement)aElements.elementAt(i)).write( pFile, pPrefix + ExceptionElement.kTab );

	pFile.println( pPrefix + "}");
    }

    /** Returns a number indicating the "size" of the exception block.
      * @return The number of atomic elements transitively contained in this
      * block.
      */
    public int getCheckSum()
    {
	int lReturn = 0;
	for( int i = 0; i < aElements.size(); i++ )
	    lReturn += ((ExceptionElement)aElements.elementAt(i)).getCheckSum();
	return lReturn;
    }
}
