/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import java.util.Set;
import java.io.PrintWriter;

/** An element that can be contained in an ExceptionBlock */
public interface ExceptionElement
{
    /** The default indentation tab used when writing out
      * Jex Files.
      */
    public static final String kTab = "   ";

    /** Returns an array of strings containing all the exception types that
     * can origin from the structure represented by this container. 
     */
    public Set getExceptions();

    /** Writes out the structure of the container onto the file specified.
      * @param pFile The PrintWriter to write the element to.
      * @param pPrefix The indentation.
      */
    public void write( PrintWriter pFile, String pPrefix );

    /** Returns true if the container is identical to the container
      * in argument.  Not necessarily the same object.
      */
    public boolean equals( Object pContainer );

    /** Returns an indication of how big the concern element
      * is.  For the purpose of fix-points analysis.
      */
    public int getCheckSum();
}
