/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import java.io.PrintWriter;

/** An ExceptionElement representing an exception
 * implicitly raised (i.e., not thrown).
 */
public class ExceptionGet extends SingleExceptionElement
{
    private String aOrigin;

    private static final String kEnv = "*environment*";
    public  static final String  kOutOfMemory = "java.lang.OutOfMemoryError";
    public  static final String  kNegativeArraySize = "java.lang.NegativeArraySizeException";
    public  static final String  kArrayIndexOutOfBounds = "java.lang.ArrayIndexOutOfBoundsException";
    public  static final String  kArithmetic = "java.lang.ArithmeticException";
    public  static final String  kNullPointer = "java.lang.NullPointerException";
    public  static final String  kArrayStore = "java.lang.ArrayStoreException";
    public  static final String  kClassCast = "java.lang.ClassCastException";

    /** Creates an implicit exception descriptor representing
      * an exception raised by a method call.
      * @param PType The class type of the exception.
      * @param pOrigin A description of what raises the
      * exception.
      */
    public ExceptionGet( Class pType, String pOrigin )
    {
	super( pType );
	aOrigin = pOrigin;
    }

    /** Creates an implicit exception descriptor representing an
      * exception raised by the environment.
      * @param pType The class type of the exception.
      */
    public ExceptionGet( Class pType )
    {
	super( pType );
	aOrigin = kEnv;
    }

    /** Returns the string describing the origin of the
      * exception.
      */
    private String getOrigin()
    { return aOrigin; }

    /** Writes out the exception descriptor to a PrintWriter
      * @param pFile The File to write the descriptor to.
      * @param pPrefix The indentation.
      */
    public void write( PrintWriter pFile, String pPrefix )
    {
	pFile.println( pPrefix + getType().getName() + ":" + aOrigin + ";"  );
    }
 
    /** Determines if pContainer is equal to this object.
      * Two ExceptionGet objects are equal if both the
      * class type of the exception and the origin are equal.
      */
    public boolean equals( Object pContainer )
    {
	boolean lReturn = false;

	if( pContainer instanceof ExceptionGet )
	{
	    if( (((ExceptionGet)pContainer).getType().equals( getType() )) &&
		(((ExceptionGet)pContainer).getOrigin().equals( aOrigin )))
		lReturn = true;
	}
	return lReturn;
    }
}
