/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import at.dms.kjc.CClass;
import java.util.Set;
import java.util.Hashtable;
import java.util.Iterator;

import java.io.*;

/** Represents the dynammic version of a Jex File
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:45 $
  */
public class JexFile
{
    protected CClass aClass;
    protected Hashtable aMethods;
    private ExceptionBlock aInit;  // Initializer block and field initialization

    
    public JexFile( CClass pClass )
    { 
	aClass = pClass;
	aMethods = new Hashtable();
	aInit = new ExceptionBlock();
    }

    public CClass getCClass()
    {
	return aClass;
    }

    /** Copies method headers only */
    public JexFile copy()
    {
	JexFile lReturn = new JexFile( aClass );
	Iterator lMethods = aMethods.values().iterator();
	while( lMethods.hasNext() )
	{
	    JexMethod lNewMethod = ((JexMethod)lMethods.next()).copy();
	    lReturn.addJexMethod( lNewMethod );
	}
	return lReturn;
    }

    public boolean isAnonymous()
    { return false; }

    public void addJexMethod( JexMethod pMethod )
    {
	aMethods.put( pMethod.getSignature(), pMethod );
    }

    public JexMethod getJexMethod( String pSignature )
    {
	return (JexMethod)aMethods.get( pSignature );
    } 

    public String getOwnerName()
    {
	CClass lOwner = null;
	CClass lTemp = aClass.getOwner();
	while( lTemp != null )
	{
	    lOwner = lTemp;
	    lTemp = lOwner.getOwner();
	}
	if( lOwner == null )
	    return null;
	else
	    return lOwner.getQualifiedName();
    }

    public ExceptionBlock getInitializerBlock()
    { return aInit; }

    public void setInitializerBlock( ExceptionBlock pInit )
    { aInit = pInit; }

    public String getQualifiedName()
    { return aClass.getQualifiedName().replace('/','.'); }

    /** Returns the exceptions for pSignature.  Mimicks the
      * semantics of the Jex Loader.
      */
    public Set getExceptions( String pSignature ) throws NoSuchMethodException
    {
	JexMethod lMethod = getJexMethod( pSignature );
	if( lMethod == null )
	    throw new NoSuchMethodException( pSignature );

	Set lReturn = lMethod.getExceptions();

	if( pSignature.startsWith("<init>" ))
	    lReturn.addAll( aInit.getExceptions() );

	return lReturn;
    }

    public void write( PrintWriter pWriter )
    {
	Iterator lMethods = aMethods.values().iterator();
	while( lMethods.hasNext() )
	{
	    ((JexMethod)lMethods.next()).write( pWriter, aInit );
	}
    }

    public int getCheckSum()
    {
	int lCheckSum = aInit.getCheckSum();
	Iterator lMethods = aMethods.values().iterator();
	while( lMethods.hasNext() )
	{
	    lCheckSum += ((JexMethod)lMethods.next()).getCheckSum();
	}
	return lCheckSum;
    }

    // For testing
    public void dump()
    {
	System.out.println();
	System.out.println( getQualifiedName() );
	Iterator lMethods = aMethods.values().iterator();
	while( lMethods.hasNext() )
	{
	    ((JexMethod)lMethods.next()).dump();
	}
    }
}
