/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import java.util.Hashtable;
import at.dms.kjc.JClassDeclaration;
import java.util.Iterator;

import java.io.*;

/** Represents a Collection of all the Jex Files and corresponding 
  * AST nodes for a Java file. 
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:45 $
  */
public class JexFileCollection
{
    private Hashtable aClassDecls;
    private Hashtable aJexFiles;

    public JexFileCollection()
    { 
	aClassDecls = new Hashtable();
	aJexFiles  = new Hashtable();
    }

    public void addClassDecl( JClassDeclaration pClass )
    {
	aClassDecls.put( pClass.getCClass().getQualifiedName(), pClass );
    }

    /** The key is the fully qualified name for the class,
      * using the slash notation. */
    public void addJexFile( String pKey, JexFile pJexFile )
    {
	aJexFiles.put( pKey, pJexFile );
    }

    public boolean contains( String pKey )
    { return aJexFiles.containsKey( pKey ); }

    public JClassDeclaration getClassDecl( String pKey )
    { 
	return (JClassDeclaration)aClassDecls.get( pKey );
    }

    public int size()
    {
	return aJexFiles.size();
    }

    public JexFile getJexFile( String pKey )
    {
	return (JexFile)aJexFiles.get( pKey );
    }

    public Iterator iterator()
    {
	return aJexFiles.values().iterator();
    }

    public int getCheckSum()
    {
	int lReturn = 0;
	Iterator lJexFiles = iterator();
	while( lJexFiles.hasNext() )
	{
	    lReturn += ((JexFile)lJexFiles.next()).getCheckSum();
	}
	return lReturn;
    }

    // For debugging purposes
    public void dump()
    {
	Iterator lIt = iterator();
	while( lIt.hasNext() )
	{
	    JexFile lJexFile = (JexFile)lIt.next();
	    //lJexFile.dump();
	    if( lJexFile.isAnonymous() )
		System.out.println( "A: " + lJexFile.getQualifiedName() );
	    else
		System.out.println( lJexFile.getQualifiedName() );
	}
    }

    public void writeJexFiles( String pPath ) throws IOException
    {
	Iterator lJexFiles = iterator();
	int i = 0;
	while( lJexFiles.hasNext() )
	{
	    JexFile lCurrent = (JexFile)lJexFiles.next();
	    String lName = "";
	    if( lCurrent instanceof AnonymousJexFile )
	    {
		lName = pPath + File.separator + lCurrent.getOwnerName() + "$$" + i + ".jex";
		i++;
	    }
	    else
	    {
		lName = pPath + File.separator + lCurrent.getQualifiedName().replace('.',File.separatorChar) + ".jex";
	    }

	    File lFile = new File( lName );
	    try
	    {
		lFile.createNewFile();
		PrintWriter lOut = new PrintWriter( new FileWriter( lFile )); 
		lCurrent.write( lOut );
		lOut.close();
	    }
	    catch( IOException e )
	    {
		throw new IOException( "IO Error: Could not create file: " + lName );
	    }
	}
    }
}
