/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import java.io.PrintWriter;
import java.util.Vector;
import java.util.Set;
import java.util.Iterator;
import java.util.HashSet;

public class TryBlock implements ExceptionElement
{
    protected ExceptionBlock aTryBlock;
    protected Vector aCatchClauses;         // Vector of Class
    protected Vector aCatchBlocks;          // Vector of ExceptionBlocks
    protected ExceptionBlock aFinallyBlock; // Should only be created if necessary
    
    public TryBlock()
    {
	aTryBlock = new ExceptionBlock();
	aFinallyBlock = null;       // Created in getFinallyBlock
	aCatchClauses = new Vector();
	aCatchBlocks = new Vector();
    }

    public boolean equals( Object pContainer )
    {
	return false;
    }

    public ExceptionBlock getTryBlock()
    {
	return aTryBlock;
    }

    public ExceptionBlock getFinallyBlock()
    {
	if( aFinallyBlock == null )
	    aFinallyBlock = new ExceptionBlock();
	return aFinallyBlock;
    }
    
    public ExceptionBlock addCatchBlock( Class pType )
    {
	aCatchClauses.addElement( pType );
	ExceptionBlock lReturn = new ExceptionBlock();
	aCatchBlocks.addElement( lReturn );
	return lReturn;
    }

    /** Returns all the exceptions that can actually make it out of the try block
     * These exceptions consist of:
     * - all the exceptions in the try block that cannot be subsumed into any of the
     * types in the catch clauses.
     * - all the exceptions produced in the catch blocks
     * - all the exception produced in the finally block;
     */
    public Set getExceptions()
    {
	HashSet lReturn = new HashSet();

	// Get the exceptions from try block
	
	Iterator lTryExceptions = aTryBlock.getExceptions().iterator();
	while( lTryExceptions.hasNext() )
	{
	    Class lCurrentException = (Class)lTryExceptions.next();
	    int j = 0;
	    while( j < aCatchClauses.size() )
	    {
		if( ((Class)aCatchClauses.elementAt(j)).isAssignableFrom( lCurrentException ))
		    break;
		j++;
	    }
	    if( j == aCatchClauses.size() )
		lReturn.add( lCurrentException );
	}

	// Add the exceptions from all the catch blocks
	for( int i = 0; i < aCatchBlocks.size(); i++ )
	{
	    lReturn.addAll( ((ExceptionBlock)aCatchBlocks.elementAt(i)).getExceptions() );
	}

	// Add the exceptions from the finally block
	if( aFinallyBlock != null )
	    lReturn.addAll( aFinallyBlock.getExceptions() );
	
	return lReturn;
    }

    public void write( PrintWriter pFile, String pPrefix )
    {
	pFile.println( pPrefix + "try");
	aTryBlock.write( pFile, pPrefix );
	for( int i = 0; i < aCatchClauses.size(); i++ )
	{
	    pFile.println( pPrefix + "catch ( " + ((Class)aCatchClauses.elementAt(i)).getName() + " )" );
	    ((ExceptionBlock)aCatchBlocks.elementAt(i)).write( pFile, pPrefix );
	}
	if( aFinallyBlock != null )
	{
	    pFile.println( pPrefix + "finally");
	    aFinallyBlock.write( pFile, pPrefix );
	}
    }

    public int getCheckSum()
    {
	int lReturn = aTryBlock.getCheckSum();
	for( int i = 0; i < aCatchBlocks.size(); i++ )
	    lReturn += ((ExceptionBlock)aCatchBlocks.elementAt(i)).getCheckSum();
	if( aFinallyBlock != null )
	    lReturn += aFinallyBlock.getCheckSum();
	return lReturn;
    }
}
