/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex;

import ca.ubc.cs.jex.util.BasicMethodRelationships;
import ca.ubc.cs.jex.util.MethodRelationships;
import ca.ubc.cs.jex.util.MethodRelationshipsException;
import java.util.Vector;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Set;
import java.util.HashSet;
import ca.ubc.cs.jex.file.*;
import java.lang.reflect.Method;
import java.util.Iterator;

/** Represents the world in which Jex operates
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:46 $
  */
public class Workspace
{
    private String[] aPackages;
    private MethodRelationships aMethodDB;
    private JexLoader aLoader;
    
    /*
    public Workspace( String pFileName ) throws IOException, MethodRelationshipsException
    {
	BufferedReader lInput = new BufferedReader( new FileReader( pFileName ));
	String lJexPath = lInput.readLine();
	
	Vector lPackages = new Vector();
	String lLine = lInput.readLine();
	while( lLine != null )
	{
	    lPackages.addElement( lLine );
	    lLine = lInput.readLine();
	}
	lInput.close();
	aPackages = new String[ lPackages.size() ];
	lPackages.copyInto( aPackages );

	// Load the method database
	aMethodDB = new BasicMethodRelationships();
	for( int i = 0; i < aPackages.length; i++ )
	    aMethodDB.addPackage( aPackages[i] );

	aLoader = new JexLoader( lJexPath );
	}*/

    public Workspace( String pJexPath, String[] pPackages ) throws MethodRelationshipsException
    {
	aPackages = pPackages;
	aMethodDB = new BasicMethodRelationships();
	for( int i = 0; i < aPackages.length; i++ )
	    aMethodDB.addPackage( aPackages[i] );

	aLoader = new JexLoader( pJexPath );
    }

    public Set getExceptionsForNonVirtualCall( String pClassName, String pSignature )
	throws NoSuchMethodException, FileNotFoundException, ParseException, ClassNotFoundException
    {
	return aLoader.getExceptionsFromSingleClass( pClassName, pSignature );
    }

    /*
    public Set getExceptionsForVirtualCall( String pClassName, String pSignature )
	throws NoSuchMethodException, FileNotFoundException, ParseException, ClassNotFoundException, IOException
    {
	HashSet lReturn = new HashSet();
	Vector lImpls = new Vector();

	try
	{
	    lImpls = aMethodDB.getImplementations( Convert.toMethod( pClassName, pSignature ));
	}
	catch( MethodRelationshipsException e )
        { // Do nothing, we simply aren't interested int the method
	}
	
	for( int i = 0; i < lImpls.size(); i++ )
	{
	    Method lMethod = (Method)lImpls.elementAt(i);
	    lReturn.addAll( getExceptionsForNonVirtualCall( lMethod.getDeclaringClass().getName(), 
							    Convert.toJexSignature( lMethod )));
	}

	// Add the anonymous classes
	for( int i = 0 ; i < aPackages.length; i++ )
	{
	    lReturn.addAll( aLoader.getExceptionsFromAnonymousClasses( aPackages[i], pClassName,  pSignature ));
	}
	return lReturn;
	}*/

    public Set getExceptionsFromAnonymousClasses( String pClassName, String pSignature )
	throws IOException, ClassNotFoundException, ParseException
    {
	HashSet lReturn = new HashSet();
	for( int i = 0; i < aPackages.length; i++ )
	{
	    lReturn.addAll( aLoader.getExceptionsFromAnonymousClasses( aPackages[i], pClassName,  pSignature ));
	}
	return lReturn;   
    }

    /** Returns a vector of java.lang.reflect.Method */
    public Vector getImplementations( String pClassName, String pSignature )
	throws ClassNotFoundException, NoSuchMethodException, MethodRelationshipsException
    {
	return aMethodDB.getImplementations( Convert.toMethod( pClassName, pSignature ));
    }

    public String getJexPath()
    {
	return aLoader.getJexPath();
    }

    public String[] getPackages()
    {
	return aPackages;
    }

    /*public static void main( String[] args ) throws Exception
    {
	Workspace lWorkspace = new Workspace( "w.txt" );
	System.out.println( lWorkspace.aLoader.getJexPath() );
	for( int i = 0; i < lWorkspace.aPackages.length; i++ )
	    System.out.println( lWorkspace.aPackages[i] );

	Set lExs = lWorkspace.getExceptionsForVirtualCall( "java.io.Writer", "write([C)" );
	Iterator lIt = lExs.iterator();
	while( lIt.hasNext() )
	    System.out.println( lIt.next() );
	    }*/
}
