/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex.file;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.FilenameFilter;
import java.util.HashSet;
import java.util.Set;

/** Reads a Jex config file
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:55:49 $
  */
public class ConfigReader
{
    private String[]  aPackages = null;
    private String    aJexPath = "";
    private String[]  aJavaFiles = null;
    private String[]  aStubs = null;
    
    private final static String kJexPath = "<jexpath>";
    private final static String kPackage = "<package>";
    private final static String kFile    = "<file>";
    private final static String kDir     = "<dir>";
    private final static String kGenStub = "<genstub>";

    public ConfigReader( File pFile ) throws IOException
    {
	BufferedReader lInput = new BufferedReader( new FileReader( pFile ));
	
	HashSet lPackages = new HashSet();
	HashSet lFiles    = new HashSet();
	HashSet lStubs     = new HashSet();
	String lLine = lInput.readLine();
	
	while( lLine != null )
	{
	    if( lLine.startsWith( kJexPath ))
		aJexPath = lLine.substring( kJexPath.length() ).trim();
	    else if( lLine.startsWith( kPackage ))
		lPackages.add( lLine.substring( kPackage.length() ).trim());
	    else if( lLine.startsWith( kFile ))
		lFiles.add( new File( lLine.substring( kFile.length() ).trim() ).getAbsolutePath());
	    else if( lLine.startsWith( kDir ))
		lFiles.addAll( getJavaFiles( lLine.substring( kDir.length() ).trim() ));
	    else if( lLine.startsWith( kGenStub ))
		lStubs.add( lLine.substring(kGenStub.length() ).trim() );	    
	    lLine = lInput.readLine();
	}

	lInput.close();

	Object[] lTemp = lPackages.toArray();
	aPackages = new String[ lTemp.length ];
	for( int i = 0; i < lTemp.length; i++ )
	    aPackages[i] = (String)lTemp[i];

	lTemp = lStubs.toArray();
	aStubs = new String[ lTemp.length ];
	for( int i = 0; i < lTemp.length; i++ )
	    aStubs[i] = (String)lTemp[i];

	lTemp = lFiles.toArray();
	aJavaFiles = new String[ lTemp.length ];
	for( int i = 0; i < lTemp.length; i++ )
	    aJavaFiles[i] = (String)lTemp[i];
    }
    
    public String getJexPath()
    { return aJexPath; }
    
    public String[] getPackages()
    { return aPackages; }

    public String[] getFiles()
    { return aJavaFiles; }

    public String[] getStubTargets()
    { return aStubs; }

    private Set getJavaFiles( String pDir ) throws IOException
    {
	File lDir = new File( pDir );
	File[] lFiles = lDir.listFiles( new JavaFilenameFilter() );
	HashSet lReturn = new HashSet();
	if( lFiles != null )
	{
	    for( int i = 0; i < lFiles.length; i++ )
		lReturn.add( lFiles[i].getCanonicalPath() );
	}
	return lReturn;
    }
    
    public static void main(String[] args) throws IOException
    {
	ConfigReader lReader = new ConfigReader( new File( args[0] ));
	System.out.println();
	System.out.println("JexPath: " + lReader.getJexPath() );
	String[] lPackages = lReader.getPackages();
	for( int i= 0; i < lPackages.length; i++ )
	    System.out.println( "Package: " + lPackages[i] );
	String[] lFiles = lReader.getFiles();
	for( int i = 0; i < lFiles.length; i++ )
	    System.out.println( "File: " + lFiles[i] );
    }
}

class JavaFilenameFilter implements FilenameFilter
{
    public boolean accept( File pDir, String pName )
    {
	return pName.endsWith(".java" );
    }
}
