/* Jex - A tool for analyzing exception flow in Java programs
 *
 *  Copyright (C) 2001 Martin P. Robillard (mrobilla@cs.ubc.ca)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ca.ubc.cs.jex.util;

import java.io.File;
import java.io.IOException;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.Vector;
import java.util.Enumeration;

/** A ZipPackageAccessor represent an object that can serve files
  * located in a zip file.
  * @author $Author: mrobilla $
  * @version $Revision: 1.2 $ $Date: 2002/03/08 21:56:23 $
  */
public class ZipPackageAccessor extends PackageAccessor
{
    protected static final String kArchivePathSeparator       = "/";
    protected static final char   kArchivePathSeparatorChar   = '/';
    protected static final String kAlternatePathSeparator     = "\\";
    protected static final char   kAlternatePathSeparatorChar = '\\';

    /** Builds a new zip accessor for file pPath.
      * @param pPath The Zip file representing a class path.
      */
    public ZipPackageAccessor( File pPath )
    {
	super( pPath );
    }

    /** Returns true if the argument is a valid zip file
      * @param pFile File to check if we can match to.
      * @return true if the argument is a valid directory
      */
    public static boolean match( File pFile )
    {
	if( pFile.getName().endsWith( FileTypesI.kZip ) )
	    return true;
	else
	    return false;
    }

    /** Returns the names of the classes contained in the package specified as argument,
      * @param pPackageName Fully qualified package name
      * @return An array of strings containing the classes in the package.  Null if the
      * package cannot be found in the current structure.
      * @exception IOException if it is impossible to open the
      * specified file.  ZipException (subtype of IOException) if the file
      * is not a zip File.
      */
    public String[] getClasses( String pPackageName ) throws IOException
    {
	ZipFile lZipFile = new ZipFile( aAccessor );
	Enumeration lEntries = lZipFile.entries();
	Vector lClasses = new Vector();
		
	while( lEntries.hasMoreElements() )
	{
	    ZipEntry lCurrentEntry = (ZipEntry)lEntries.nextElement();
	    
	    if( !lCurrentEntry.isDirectory() )
	    {
		String lFileName = lCurrentEntry.getName();
				
		if( lFileName.endsWith( FileTypesI.kClass ) )
		{
		    int lLastBreak = lFileName.lastIndexOf( kArchivePathSeparator );
		    String lPackageName = "";
		    if( lLastBreak < 0 )
		    {
			lLastBreak = lFileName.lastIndexOf( kAlternatePathSeparator );
			if( lLastBreak < 0 )
			    lPackageName = "";
			else
			    lPackageName = (lFileName.substring( 0, lLastBreak )).replace( kAlternatePathSeparatorChar, '.');
		    }
		    else
		    {
			lPackageName = (lFileName.substring( 0, lLastBreak )).replace( kArchivePathSeparatorChar, '.');
		    }

		    String lClassName = lFileName.substring( lLastBreak + 1, lFileName.length() );
		    lClassName = lClassName.substring( 0, lClassName.length() - 6 );
		    		    
		    if( lPackageName.equals( pPackageName ) )
		    {
			lClasses.addElement( lClassName );
		    }
		    else
			;// We just don't add it to the list
		}
		else
		    ; //It's not a class, we don't add it to the list.
	    }
	    else
		; // It's not a file, we don't add it to the list
	}

	String[] lReturn = null;
	if( lClasses.size() > 0 )
	{
	    lReturn = new String[ lClasses.size() ];
	    for( int i = 0; i < lClasses.size(); i++ )
		lReturn[i] = (String)lClasses.elementAt(i);
	}

	return lReturn;
    }
}	    
